import numpy as np
import scipy.stats as ss
import matplotlib.pyplot as plt
import seaborn as sb
import pystan
import sys
 

def CalcMAPEst(samples, a = 0.05, n_points = 10000):
    """
        Calculatte a MAP estimate from a KDE graph on [Lp, Up]
        Lp and Up are 100*a/2 and 100(1-a/2) percentile points of samples
    """
    Lp, Up = np.percentile(samples, [100 * a/2, 100 * (1 - a/2)])  # import numpy as np
    coord = np.linspace(Lp, Up, n_points)
    est_pdf = ss.gaussian_kde(samples).pdf(coord)  # import scipy.stats as ss
    map_idx = np.argmax(est_pdf)
    MAP_Est = coord[map_idx]      
    return MAP_Est, est_pdf[map_idx]


n_n = [int(v) for v in input('n_n = ').split()]
K = len(n_n)
if K < 3:
    print('Number of categories should be larger than 2.')
    sys.exit()

n_s = [int(v) for v in input('n_s = ').split()]
if len(n_s) != K:
    print('Numbers of categories are not consistent.')
    sys.exit()
s_n = [int(v) for v in input('s_n = ').split()]
if len(s_n) != K:
    print('Numbers of categories are not consistent.')
    sys.exit()
s_s = [int(v) for v in input('s_s = ').split()]
if len(s_s) != K:
    print('Numbers of categories are not consistent.')
    sys.exit()
print('n_n =', n_n)
print('n_s =', n_s)
print('s_n =', s_n)
print('s_s =', s_s)

sm = pystan.StanModel(file = 'SDT_2AFC_KCat_odd.stan' if K % 2 == 1 else 'SDT_2AFC_KCat_even.stan')

print('Compiling ended.')

Data = {'K':K, 'nn_cond':n_n, 'ns_cond':n_s, 'sn_cond':s_n, 'ss_cond':s_s}
fit = sm.sampling(data = Data, iter = 10000, n_jobs = 1)
print(fit)

mu_s_map = CalcMAPEst(fit['mu_s'])[0]
sb.kdeplot(fit['mu_s'])
plt.xlabel('$\mu_s$')
plt.title('$\mu_s$(MAP) = {0:.3f}'.format(mu_s_map))
plt.savefig('Fig_mu.png')
plt.show()

sgm_s_map = CalcMAPEst(fit['sgm_s'])[0]
sb.kdeplot(fit['sgm_s'])
plt.title('$\sigma_s$(MAP) = {0:.3f}'.format(sgm_s_map))
plt.xlabel('$\sigma_s$')
plt.savefig('Fig_sgm.png')
plt.show()



s_title = ''
if K % 2 == 1:
    for k in range(K-1):
        sb.kdeplot(fit['C'].T[k])
        c_map = CalcMAPEst(fit['C'].T[k])[0]
        s_title += f'C{k+1}={c_map:.2f}'
        if k < K-2:
            s_title += ', '
    
else:
    for k in range(K-1):
        if k != (K//2) - 1:
            sb.kdeplot(fit['C'].T[k])
            c_map = CalcMAPEst(fit['C'].T[k])[0]
            s_title += f'C{k+1}={c_map:.2f}'
            if k < K-2:
                s_title += ', '
        else:
            plt.plot([0], [0], marker = 'o', markersize = 15, c = 'r')
            s_title += f'C{k+1}=0, '
            
plt.title('MAP estimates\n' + s_title)
plt.savefig('FigC.png')
plt.show()

b_map = CalcMAPEst(fit['b'])[0]
sb.kdeplot(fit['b'])
plt.title('b(MAP) = {0:.3f}'.format(b_map))
plt.savefig('Fig_b.png')
plt.show()


cum_n_n = np.cumsum(n_n)
pcum_n_n = cum_n_n/cum_n_n[-1]
cum_n_s = np.cumsum(n_s)
pcum_n_s = cum_n_s / cum_n_s[-1]
cum_s_n = np.cumsum(s_n)
pcum_s_n = cum_s_n / cum_s_n[-1]
cum_s_s = np.cumsum(s_s)
pcum_s_s = cum_s_s / cum_s_s[-1]

est_pcum_n_n = np.median(fit['nn_cum_p'], axis = 0)[1:-1]
est_pcum_n_s = np.median(fit['ns_cum_p'], axis = 0)[1:-1]
est_pcum_s_n = np.median(fit['sn_cum_p'], axis = 0)[1:-1]
est_pcum_s_s = np.median(fit['ss_cum_p'], axis = 0)[1:-1]

plt.plot(est_pcum_n_n, pcum_n_n[:-1], label = 'n_n')
plt.plot(est_pcum_n_s, pcum_n_s[:-1], label = 'n_s')
plt.plot(est_pcum_s_n, pcum_s_n[:-1], label = 's_n')
plt.plot(est_pcum_s_s, pcum_s_s[:-1], label = 's_s')
plt.plot([0,1], [0,1], c = 'k', ls = '--', label = 'Obs.=Est.')
plt.xlabel('Est.Cum.P')
plt.ylabel('Obs.Cum.P')
plt.legend()
plt.title('Cumulative Proportions')
plt.savefig('FigCumEstObs.png')
plt.show()
